from pydantic import BaseModel, Field


class SignalData(BaseModel):
    """The data to send in a signal.

    Args:
        type (str): The type of data being sent to the client.
        data (str): Payload to send to the client.
    """

    type: str = Field(..., max_length=128)
    data: str = Field(..., max_length=8192)



from typing import Optional

from pydantic import BaseModel, Field, model_validator
from vonage_video.errors import LayoutScreenshareTypeError, LayoutStylesheetError
from vonage_video.models.enums import LayoutType


class VideoStream(BaseModel):
    """Model for a video stream used for archive and broadcast operations.

    Args:
        stream_id (str, Optional): The stream ID.
        has_audio (bool, Optional): Whether the stream has audio.
        has_video (bool, Optional): Whether the stream has video.
    """

    stream_id: Optional[str] = Field(None, validation_alias='streamId')
    has_audio: Optional[bool] = Field(None, validation_alias='hasAudio')
    has_video: Optional[bool] = Field(None, validation_alias='hasVideo')


class AddStreamRequest(BaseModel):
    """Model for adding a stream to an archive or broadcast.

    Args:
        stream_id (VideoStream): The stream ID to add to the archive/broadcast.
        has_audio (bool, Optional): Whether the stream has audio.
        has_video (bool, Optional): Whether the stream has video.
    """

    stream_id: str = Field(..., serialization_alias='addStream')
    has_audio: Optional[bool] = Field(None, serialization_alias='hasAudio')
    has_video: Optional[bool] = Field(None, serialization_alias='hasVideo')


class ComposedLayout(BaseModel):
    """Model for layout options for a composed archive/broadcast.

    Args:
        type (str): Specify this to assign the initial layout type for the archive/broadcast.
            This applies only to composed archives.
        stylesheet (str, Optional): The stylesheet URL. Used for the custom layout to
            define the visual layout.
        screenshare_type (str, Optional): The screenshare type. Set the screenshareType
            property to the layout type to use when there is a screen-sharing stream in
            the session. If you set the screenshareType property, you must set the type
            property to "bestFit" and leave the stylesheet property unset.

    Raises:
        LayoutStylesheetError: If `stylesheet` is not set for `layout_type: 'custom'` or
            if `stylesheet` is set for `layout_type: 'bestFit'`.
        LayoutScreenshareTypeError: If `screenshare_type` is set and `type` is not 'bestFit'.
    """

    type: LayoutType
    stylesheet: Optional[str] = None
    screenshare_type: Optional[LayoutType] = Field(
        None, serialization_alias='screenshareType'
    )

    @model_validator(mode='after')
    def validate_stylesheet(self):
        if self.type == LayoutType.CUSTOM and self.stylesheet is None:
            raise LayoutStylesheetError(
                'The `stylesheet` property must be set for `layout_type: \'custom\'`.'
            )
        if self.type != LayoutType.CUSTOM and self.stylesheet is not None:
            raise LayoutStylesheetError(
                'The `stylesheet` property cannot be set for `layout_type: \'bestFit\'`.'
            )
        return self

    @model_validator(mode='after')
    def type_and_screenshare_type(self):
        if self.screenshare_type is not None and self.type != LayoutType.BEST_FIT:
            raise LayoutScreenshareTypeError(
                'If `screenshare_type` is set, `type` must have the value `bestFit`.'
            )
        return self


class ListVideoFilter(BaseModel):
    """Base model to filter when listing archives/broadcasts/Experience Composers.

    Args:
        offset (int, Optional): The offset.
        page_size (int, Optional): The number of archives to return per page.
    """

    offset: Optional[int] = None
    page_size: Optional[int] = Field(100, serialization_alias='count')



from typing import Optional

from pydantic import BaseModel, Field
from vonage_video.models.enums import AudioSampleRate


class AudioConnectorWebSocket(BaseModel):
    """The audio connector websocket options.

    Args:
        uri (str): The URI.
        streams (list[str]): Stream IDs to include. If not provided, all streams are included.
        headers (dict): The headers to send to your WebSocket server.
        audio_rate (AudioSampleRate): The audio sample rate in Hertz.
    """

    uri: str
    streams: Optional[list[str]] = None
    headers: Optional[dict] = None
    audio_rate: Optional[AudioSampleRate] = Field(None, serialization_alias='audioRate')


class AudioConnectorOptions(BaseModel):
    """Options for the audio connector.

    Args:
        session_id (str): The session ID.
        token (str): The token.
        websocket (AudioConnectorWebSocket): The audio connector websocket.
    """

    session_id: str = Field(..., serialization_alias='sessionId')
    token: str
    websocket: AudioConnectorWebSocket


class AudioConnectorData(BaseModel):
    """Class containing Audio Connector WebSocket ID and connection ID.

    Args:
        id (str, Optional): The WebSocket ID.
        connection_id (str, Optional): The connection ID.
    """

    id: Optional[str] = None
    connection_id: Optional[str] = Field(None, validation_alias='connectionId')



from enum import Enum


class TokenRole(str, Enum):
    """The role assigned to the token."""

    SUBSCRIBER = 'subscriber'
    PUBLISHER = 'publisher'
    PUBLISHER_ONLY = 'publisheronly'
    MODERATOR = 'moderator'


class ArchiveMode(str, Enum):
    """Whether the session is archived automatically ("always") or not ("manual")."""

    MANUAL = 'manual'
    ALWAYS = 'always'


class MediaMode(str, Enum):
    """Whether the session uses the Vonage Video media router ("routed") or peers connect
    directly (relayed)."""

    ROUTED = 'routed'
    RELAYED = 'relayed'


class P2pPreference(str, Enum):
    """The preference for peer-to-peer connections."""

    DISABLED = 'disabled'
    ALWAYS = 'always'


class LanguageCode(str, Enum):
    EN_US = 'en-US'
    EN_AU = 'en-AU'
    EN_GB = 'en-GB'
    ZH_CN = 'zh-CN'
    FR_FR = 'fr-FR'
    FR_CA = 'fr-CA'
    DE_DE = 'de-DE'
    HI_IN = 'hi-IN'
    IT_IT = 'it-IT'
    JA_JP = 'ja-JP'
    KO_KR = 'ko-KR'
    PT_BR = 'pt-BR'
    TH_TH = 'th-TH'


class AudioSampleRate(int, Enum):
    """Audio sample rate, in Hertz."""

    KHZ_8 = 8000
    KHZ_16 = 16000


class VideoResolution(str, Enum):
    """The resolution of the archive or broadcast.

    This property only applies to composed archives. If you set this property and set the
    outputMode property to "individual", the call to the REST method results in an error.
    """

    RES_640x480 = '640x480'
    RES_480x640 = '480x640'
    RES_1280x720 = '1280x720'
    RES_720x1280 = '720x1280'
    RES_1920x1080 = '1920x1080'
    RES_1080x1920 = '1080x1920'


class ExperienceComposerStatus(str, Enum):
    STARTING = 'starting'
    STARTED = 'started'
    STOPPED = 'stopped'
    FAILED = 'failed'


class OutputMode(str, Enum):
    COMPOSED = 'composed'
    INDIVIDUAL = 'individual'


class StreamMode(str, Enum):
    """Whether streams included in the archive are selected automatically ("auto", the
    default) or manually ("manual")."""

    AUTO = 'auto'
    MANUAL = 'manual'


class LayoutType(str, Enum):
    BEST_FIT = 'bestFit'
    CUSTOM = 'custom'
    PIP = 'pip'
    VERTICAL_PRESENTATION = 'verticalPresentation'
    HORIZONTAL_PRESENTATION = 'horizontalPresentation'


class ArchiveStatus(str, Enum):
    AVAILABLE = 'available'
    EXPIRED = 'expired'
    FAILED = 'failed'
    PAUSED = 'paused'
    STARTED = 'started'
    STOPPED = 'stopped'
    UPLOADED = 'uploaded'
    DELETED = 'deleted'



from typing import Optional, Type, Union

from pydantic import validate_call
from vonage_http_client.errors import HttpRequestError
from vonage_http_client.http_client import HttpClient
from vonage_utils.types import Dtmf
from vonage_video.errors import (
    InvalidArchiveStateError,
    InvalidBroadcastStateError,
    RoutedSessionRequiredError,
    VideoError,
)
from vonage_video.models.archive import (
    Archive,
    ComposedLayout,
    CreateArchiveRequest,
    ListArchivesFilter,
)
from vonage_video.models.audio_connector import AudioConnectorData, AudioConnectorOptions
from vonage_video.models.broadcast import (
    Broadcast,
    CreateBroadcastRequest,
    ListBroadcastsFilter,
)
from vonage_video.models.captions import CaptionsData, CaptionsOptions
from vonage_video.models.common import AddStreamRequest
from vonage_video.models.experience_composer import (
    ExperienceComposer,
    ExperienceComposerOptions,
    ListExperienceComposersFilter,
)
from vonage_video.models.session import SessionOptions, VideoSession
from vonage_video.models.signal import SignalData
from vonage_video.models.sip import InitiateSipRequest, SipCall
from vonage_video.models.stream import StreamInfo, StreamLayoutOptions
from vonage_video.models.token import TokenOptions


class Video:
    """Calls Vonage's Video API."""

    def __init__(self, http_client: HttpClient) -> None:
        self._http_client = http_client

    @property
    def http_client(self) -> HttpClient:
        """The HTTP client used to make requests to the Voice API.

        Returns:
            HttpClient: The HTTP client used to make requests to the Voice API.
        """
        return self._http_client

    @validate_call
    def generate_client_token(self, token_options: TokenOptions) -> bytes:
        """Generates a client token for the Vonage Video API.

        Args:
            token_options (TokenOptions): The options for the token.

        Returns:
            str: The client token.
        """
        return self._http_client.auth.generate_application_jwt(
            token_options.model_dump(exclude_none=True)
        )

    @validate_call
    def create_session(self, options: SessionOptions = None) -> VideoSession:
        """Creates a new session for the Vonage Video API.

        Args:
            options (SessionOptions): The options for the session.

        Returns:
            VideoSession: The new session ID, plus the config options specified in `options`.
        """

        response = self._http_client.post(
            self._http_client.video_host,
            '/session/create',
            options.model_dump(by_alias=True, exclude_none=True) if options else None,
            sent_data_type='form',
        )

        session_response = {
            'session_id': response[0]['session_id'],
            **(options.model_dump(exclude_none=True) if options else {}),
        }

        return VideoSession(**session_response)

    @validate_call
    def list_streams(self, session_id: str) -> list[StreamInfo]:
        """Lists the streams in a session from the Vonage Video API.

        Args:
            session_id (str): The session ID.

        Returns:
            list[StreamInfo]: Information about the video streams.
        """

        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/stream',
        )

        return [StreamInfo(**stream) for stream in response['items']]

    @validate_call
    def get_stream(self, session_id: str, stream_id: str) -> StreamInfo:
        """Gets a stream from the Vonage Video API.

        Args:
            session_id (str): The session ID.
            stream_id (str): The stream ID.

        Returns:
            StreamInfo: Information about the video stream.
        """

        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/stream/{stream_id}',
        )

        return StreamInfo(**response)

    @validate_call
    def change_stream_layout(
        self, session_id: str, stream_layout_options: StreamLayoutOptions
    ) -> list[StreamInfo]:
        """Changes the layout of a stream in a session in the Vonage Video API.

        Args:
            session_id (str): The session ID.
            stream_layout_options (StreamLayoutOptions): The options for the stream layout.

        Returns:
            list[StreamInfo]: Information about the video streams.
        """

        response = self._http_client.put(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/stream',
            stream_layout_options.model_dump(by_alias=True, exclude_none=True),
        )

        return [StreamInfo(**stream) for stream in response['items']]

    @validate_call
    def send_signal(
        self, session_id: str, data: SignalData, connection_id: str = None
    ) -> None:
        """Sends a signal to a session in the Vonage Video API. If `connection_id` is not
        provided, the signal will be sent to all connections in the session.

        Args:
            session_id (str): The session ID.
            data (SignalData): The data to send in the signal.
            connection_id (str, Optional): The connection ID to send the signal to. If not provided,
                the signal will be sent to all connections in the session.
        """
        if connection_id is not None:
            url = f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/connection/{connection_id}/signal'
        else:
            url = f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/signal'

        self._http_client.post(
            self._http_client.video_host, url, data.model_dump(exclude_none=True)
        )

    @validate_call
    def disconnect_client(self, session_id: str, connection_id: str) -> None:
        """Disconnects a client from a session in the Vonage Video API.

        Args:
            session_id (str): The session ID.
            connection_id (str): The connection ID of the client to disconnect.
        """
        self._http_client.delete(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/connection/{connection_id}',
        )

    @validate_call
    def mute_stream(self, session_id: str, stream_id: str) -> None:
        """Mutes a stream in a session using the Vonage Video API.

        Args:
            session_id (str): The session ID.
            stream_id (str): The stream ID.
        """
        self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/stream/{stream_id}/mute',
        )

    @validate_call
    def mute_all_streams(
        self, session_id: str, excluded_stream_ids: list[str] = None
    ) -> None:
        """Mutes all streams in a session using the Vonage Video API.

        Args:
            session_id (str): The session ID.
            excluded_stream_ids (list[str], Optional): The stream IDs to exclude from muting.
        """
        params = {'active': True, 'excludedStreamIds': excluded_stream_ids}
        self._toggle_mute_all_streams(session_id, params)

    @validate_call
    def disable_mute_all_streams(self, session_id: str) -> None:
        """Disables muting all streams in a session using the Vonage Video API.

        Args:
            session_id (str): The session ID.
        """
        self._toggle_mute_all_streams(session_id, {'active': False})

    @validate_call
    def _toggle_mute_all_streams(self, session_id: str, params: dict) -> None:
        """Mutes all streams in a session using the Vonage Video API.

        Args:
            session_id (str): The session ID.
            params (dict): The parameters to send in the request.
        """
        self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/mute',
            params,
        )

    @validate_call
    def start_captions(self, options: CaptionsOptions) -> CaptionsData:
        """Enables captions in a session using the Vonage Video API.

        Args:
            options (CaptionsOptions): Options for the captions.

        Returns:
            CaptionsData: Class containing captions ID.
        """
        response = self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/captions',
            options.model_dump(exclude_none=True, by_alias=True),
        )

        return CaptionsData(captions_id=response['captionsId'])

    @validate_call
    def stop_captions(self, captions: CaptionsData) -> None:
        """Disables captions in a session using the Vonage Video API.

        Args:
            captions (CaptionsData): The captions data.
        """
        self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/captions/{captions.captions_id}/stop',
        )

    @validate_call
    def start_audio_connector(self, options: AudioConnectorOptions) -> AudioConnectorData:
        """Starts an audio connector in a session using the Vonage Video API. Connects
        audio streams to a specified WebSocket URI.

        Args:
            options (AudioConnectorOptions): Options for the audio connector.

        Returns:
            AudioConnectorData: Class containing audio connector ID.
        """
        response = self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/connect',
            options.model_dump(exclude_none=True, by_alias=True),
        )

        return AudioConnectorData(**response)

    @validate_call
    def start_experience_composer(
        self, options: ExperienceComposerOptions
    ) -> ExperienceComposer:
        """Starts an Experience Composer using the Vonage Video API.

        Args:
            options (ExperienceComposerOptions): Options for the Experience Composer.

        Returns:
            ExperienceComposer: Class containing Experience Composer data.
        """

        response = self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/render',
            options.model_dump(exclude_none=True, by_alias=True),
        )

        return ExperienceComposer(**response)

    @validate_call
    def list_experience_composers(
        self, filter: ListExperienceComposersFilter = ListExperienceComposersFilter()
    ) -> tuple[list[ExperienceComposer], int, Optional[int]]:
        """Lists Experience Composers associated with your Vonage application.

        Args:
            filter (ListExperienceComposersFilter, Optional): Filter for the Experience Composers.

        Returns:
            tuple[list[ExperienceComposer], int, Optional[int]]: A tuple containing a list of experience
                composer objects, the total count of Experience Composers and the required offset value
                for the next page, if applicable.
                i.e.
                experience_composers: list[ExperienceComposer], count: int, next_page_offset: Optional[int]
        """
        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/render',
            filter.model_dump(exclude_none=True, by_alias=True),
        )

        return self._list_video_objects(filter, response, ExperienceComposer)

    @validate_call
    def get_experience_composer(self, experience_composer_id: str) -> ExperienceComposer:
        """Gets an Experience Composer associated with your Vonage application.

        Args:
            experience_composer_id (str): The ID of the Experience Composer.

        Returns:
            ExperienceComposer: The Experience Composer object.
        """
        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/render/{experience_composer_id}',
        )

        return ExperienceComposer(**response)

    @validate_call
    def stop_experience_composer(self, experience_composer_id: str) -> None:
        """Stops an Experience Composer associated with your Vonage application.

        Args:
            experience_composer_id (str): The ID of the Experience Composer.
        """
        self._http_client.delete(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/render/{experience_composer_id}',
        )

    @validate_call
    def list_archives(
        self, filter: ListArchivesFilter = ListArchivesFilter()
    ) -> tuple[list[Archive], int, Optional[int]]:
        """Lists archives associated with a Vonage Application.

        Args:
            filter (ListArchivesFilter, Optional): The filters for the archives.

        Returns:
            tuple[list[Archive], int, Optional[int]]: A tuple containing a list of archive objects,
                the total count of archives and the required offset value for the next page, if applicable.
                i.e.
                archives: list[Archive], count: int, next_page_offset: Optional[int]
        """
        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/archive',
            filter.model_dump(exclude_none=True, by_alias=True),
        )

        return self._list_video_objects(filter, response, Archive)

    @validate_call
    def start_archive(self, options: CreateArchiveRequest) -> Archive:
        """Starts an archive in a Vonage Video API session.

        Args:
            options (CreateArchiveRequest): The options for the archive.

        Returns:
            Archive: The archive object.
        """
        response = self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/archive',
            options.model_dump(exclude_none=True, by_alias=True),
        )

        return Archive(**response)

    @validate_call
    def get_archive(self, archive_id: str) -> Archive:
        """Gets an archive from the Vonage Video API.

        Args:
            archive_id (str): The archive ID.

        Returns:
            Archive: The archive object.
        """
        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/archive/{archive_id}',
        )

        return Archive(**response)

    @validate_call
    def delete_archive(self, archive_id: str) -> None:
        """Deletes an archive from the Vonage Video API.

        Args:
            archive_id (str): The archive ID.

        Raises:
            InvalidArchiveStateError: If the archive has a status other than `available`, `uploaded`, or `deleted`.
        """
        try:
            self._http_client.delete(
                self._http_client.video_host,
                f'/v2/project/{self._http_client.auth.application_id}/archive/{archive_id}',
            )
        except HttpRequestError as e:
            conflict_error_message = 'You can only delete an archive that has one of the following statuses: `available` OR `uploaded` OR `deleted`.'
            self._check_conflict_error(
                e, InvalidArchiveStateError, conflict_error_message
            )

    @validate_call
    def add_stream_to_archive(self, archive_id: str, params: AddStreamRequest) -> None:
        """Adds a stream to an archive in the Vonage Video API. Use this method to change
        the streams included in a composed archive that was started with the streamMode
        set to "manual".

        Args:
            archive_id (str): The archive ID.
            params (AddStreamRequest): Params for adding a stream to an archive.
        """
        self._http_client.patch(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/archive/{archive_id}/streams',
            params.model_dump(exclude_none=True, by_alias=True),
        )

    @validate_call
    def remove_stream_from_archive(self, archive_id: str, stream_id: str) -> None:
        """Removes a stream from an archive in the Vonage Video API.

        Args:
            archive_id (str): The archive ID.
            stream_id (str): ID of the stream to remove.
        """
        self._http_client.patch(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/archive/{archive_id}/streams',
            params={'removeStream': stream_id},
        )

    @validate_call
    def stop_archive(self, archive_id: str) -> Archive:
        """Stops a Vonage Video API archive.

        Args:
            archive_id (str): The archive ID.

        Returns:
            Archive: The archive object.

        Raises:
            InvalidArchiveStateError: If the archive is not being recorded.
        """
        try:
            response = self._http_client.post(
                self._http_client.video_host,
                f'/v2/project/{self._http_client.auth.application_id}/archive/{archive_id}/stop',
            )
        except HttpRequestError as e:
            conflict_error_message = (
                'You can only stop an archive that is being recorded.'
            )
            self._check_conflict_error(
                e, InvalidArchiveStateError, conflict_error_message
            )
        return Archive(**response)

    @validate_call
    def change_archive_layout(self, archive_id: str, layout: ComposedLayout) -> Archive:
        """Changes the layout of an archive in the Vonage Video API.

        Args:
            archive_id (str): The archive ID.
            layout (ComposedLayout): The layout to change to.

        Returns:
            Archive: The archive object.
        """
        response = self._http_client.put(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/archive/{archive_id}/layout',
            layout.model_dump(exclude_none=True, by_alias=True),
        )

        return Archive(**response)

    @validate_call
    def list_broadcasts(
        self, filter: ListBroadcastsFilter = ListBroadcastsFilter()
    ) -> tuple[list[Broadcast], int, Optional[int]]:
        """Lists broadcasts associated with a Vonage Application.

        Args:
            filter (ListBroadcastsFilter, Optional): The filters for the broadcasts.

        Returns:
            tuple[list[Broadcast], int, Optional[int]]: A tuple containing a list of broadcast objects,
                the total count of broadcasts and the required offset value for the next page, if applicable.
                i.e.
                broadcasts: list[Broadcast], count: int, next_page_offset: Optional[int]
        #
        """
        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/broadcast',
            filter.model_dump(exclude_none=True, by_alias=True),
        )

        return self._list_video_objects(filter, response, Broadcast)

    @validate_call
    def start_broadcast(self, options: CreateBroadcastRequest) -> Broadcast:
        """Starts a broadcast in a Vonage Video API session.

        Args:
            options (CreateBroadcastRequest): The options for the broadcast.

        Returns:
            Broadcast: The broadcast object.

        Raises:
            InvalidBroadcastStateError: If the broadcast has already started for the session,
                or if you attempt to start a simultaneous broadcast for a session without setting
                a unique `multi-broadcast-tag` value.
        """
        try:
            response = self._http_client.post(
                self._http_client.video_host,
                f'/v2/project/{self._http_client.auth.application_id}/broadcast',
                options.model_dump(exclude_none=True, by_alias=True),
            )
        except HttpRequestError as e:
            conflict_error_message = (
                'Either the broadcast has already started for the session, '
                'or you attempted to start a simultaneous broadcast for a session '
                'without setting a unique `multi-broadcast-tag` value.'
            )
            self._check_conflict_error(
                e, InvalidBroadcastStateError, conflict_error_message
            )

        return Broadcast(**response)

    @validate_call
    def get_broadcast(self, broadcast_id: str) -> Broadcast:
        """Gets a broadcast from the Vonage Video API.

        Args:
            broadcast_id (str): The broadcast ID.

        Returns:
            Broadcast: The broadcast object.
        """
        response = self._http_client.get(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/broadcast/{broadcast_id}',
        )

        return Broadcast(**response)

    @validate_call
    def stop_broadcast(self, broadcast_id: str) -> Broadcast:
        """Stops a Vonage Video API broadcast.

        Args:
            broadcast_id (str): The broadcast ID.

        Returns:
            Broadcast: The broadcast object.
        """
        response = self._http_client.post(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/broadcast/{broadcast_id}/stop',
        )
        return Broadcast(**response)

    @validate_call
    def change_broadcast_layout(
        self, broadcast_id: str, layout: ComposedLayout
    ) -> Broadcast:
        """Changes the layout of a broadcast in the Vonage Video API.

        Args:
            broadcast_id (str): The broadcast ID.
            layout (ComposedLayout): The layout to change to.

        Returns:
            Broadcast: The broadcast object.
        """
        response = self._http_client.put(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/broadcast/{broadcast_id}/layout',
            layout.model_dump(exclude_none=True, by_alias=True),
        )

        return Broadcast(**response)

    @validate_call
    def add_stream_to_broadcast(
        self, broadcast_id: str, params: AddStreamRequest
    ) -> None:
        """Adds a stream to a broadcast in the Vonage Video API. Use this method to change
        the streams included in a composed broadcast that was started with the streamMode
        set to "manual".

        Args:
            broadcast_id (str): The broadcast ID.
            params (AddStreamRequest): The video stream to add to the broadcast.
        """
        self._http_client.patch(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/broadcast/{broadcast_id}/streams',
            params.model_dump(exclude_none=True, by_alias=True),
        )

    @validate_call
    def remove_stream_from_broadcast(self, broadcast_id: str, stream_id: str) -> None:
        """Removes a stream from a broadcast in the Vonage Video API.

        Args:
            broadcast_id (str): The broadcast ID.
            stream_id (str): ID of the stream to remove.
        """
        self._http_client.patch(
            self._http_client.video_host,
            f'/v2/project/{self._http_client.auth.application_id}/broadcast/{broadcast_id}/streams',
            params={'removeStream': stream_id},
        )

    @validate_call
    def initiate_sip_call(self, sip_request_params: InitiateSipRequest) -> SipCall:
        """Initiates a SIP call using the Vonage Video API.

        Args:
            sip_request_params (SipParams): Model containing the session ID and a valid token,
                as well as options for the SIP call.

        Returns:
            SipCall: The SIP call object.
        """
        try:
            response = self._http_client.post(
                self._http_client.video_host,
                f'/v2/project/{self._http_client.auth.application_id}/dial',
                sip_request_params.model_dump(exclude_none=True, by_alias=True),
            )
        except HttpRequestError as e:
            conflict_error_message = 'SIP calling can only be used in a session with'
            ' `media_mode=routed`.'
            self._check_conflict_error(
                e, RoutedSessionRequiredError, conflict_error_message
            )

        return SipCall(**response)

    @validate_call
    def play_dtmf(self, session_id: str, digits: Dtmf, connection_id: str = None) -> None:
        """Plays DTMF tones into one or all SIP connections in a session using the Vonage
        Video API.

        Args:
            session_id (str): The session ID.
            digits (Dtmf): The DTMF digits to play. Numbers `0-9`, `*`, `#` and `p`
                (500ms pause) are supported.
            connection_id (str, Optional): The connection ID to send the DTMF tones to.
                If not provided, the DTMF tones will be played on all connections in
                the session.
        """
        if connection_id is not None:
            url = f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/connection/{connection_id}/play-dtmf'
        else:
            url = f'/v2/project/{self._http_client.auth.application_id}/session/{session_id}/play-dtmf'

        self._http_client.post(self._http_client.video_host, url, {'digits': digits})

    @validate_call
    def _list_video_objects(
        self,
        request_filter: Union[
            ListArchivesFilter, ListBroadcastsFilter, ListExperienceComposersFilter
        ],
        response: dict,
        model: Union[Type[Archive], Type[Broadcast], Type[ExperienceComposer]],
    ) -> tuple[list[object], int, Optional[int]]:
        """List objects of a specific model from a response.

        Args:
            request_filter (Union[ListArchivesFilter, ListBroadcastsFilter, ListExperienceComposersFilter]):
                The filter used to make the request.
            response (dict): The response from the API.
            model (Union[Type[Archive], Type[Broadcast], Type[ExperienceComposer]]): The type of a pydantic
                model to populate the response into.

        Returns:
            tuple[list[object], int, Optional[int]]: A tuple containing a list of objects,
                the total count of objects and the required offset value for the next page, if applicable.
                i.e.
                objects: list[object], count: int, next_page_offset: Optional[int]
        """
        index = request_filter.offset + 1 or 1
        page_size = request_filter.page_size
        objects = []

        try:
            for obj in response['items']:
                objects.append(model(**obj))
        except KeyError:
            return [], 0, None

        count = response['count']
        if count > page_size * index:
            return objects, count, index
        return objects, count, None

    def _check_conflict_error(
        self,
        http_error: HttpRequestError,
        ConflictError: Type[VideoError],
        conflict_error_message: str,
    ) -> None:
        """Checks if the error is a conflict error and raises the specified error.

        Args:
            http_error (HttpRequestError): The error to check.
            ConflictError (Type[VideoError]): The error to raise if there is a conflict.
            conflict_error_message (str): The error message if there is a conflict.
        """
        if http_error.response.status_code == 409:
            raise ConflictError(f'{conflict_error_message} {http_error.response.text}')
        raise http_error



from typing import Optional

from pydantic import BaseModel, Field, model_validator
from vonage_video.errors import InvalidHlsOptionsError, InvalidOutputOptionsError
from vonage_video.models.common import ComposedLayout, ListVideoFilter, VideoStream
from vonage_video.models.enums import StreamMode, VideoResolution


class ListBroadcastsFilter(ListVideoFilter):
    """Model with filters for listing broadcasts.

    Args:
        offset (int, Optional): The offset.
        page_size (int, Optional): The number of broadcast objects to return per page.
        session_id (str, Optional): The session ID of a Vonage Video session.
    """

    session_id: Optional[str] = None


class BroadcastHls(BaseModel):
    """Model for HLS output settings for a broadcast.

    Args:
        dvr (bool, Optional): Whether the broadcast supports DVR.
        low_latency (bool, Optional): Whether the broadcast is low latency.
            Note: Cannot be True when `dvr=True`.

    Raises:
        InvalidHlsOptionsError: If `low_latency=True` and `dvr=True`.
    """

    dvr: Optional[bool] = None
    low_latency: Optional[bool] = Field(None, serialization_alias='lowLatency')

    @model_validator(mode='after')
    def validate_low_latency(self):
        if self.dvr and self.low_latency:
            raise InvalidHlsOptionsError('Cannot set `low_latency=True` when `dvr=True`.')
        return self


class BroadcastRtmp(BaseModel):
    """Model for RTMP output settings for a broadcast.

    Args:
        id (str, Optional): A unique ID for the stream.
        server_url (str): The RTMP server URL.
        stream_name (str): The stream name, such as the YouTube Live stream name or the
            Facebook stream key.
    """

    id: Optional[str] = None
    server_url: str = Field(..., serialization_alias='serverUrl')
    stream_name: str = Field(..., serialization_alias='streamName')


class RtmpStream(BroadcastRtmp):
    """Model for RTMP output settings for a broadcast.

    Args:
        id (str, Optional): A unique ID for the stream.
        server_url (str): The RTMP server URL.
        stream_name (str): The stream name, such as the YouTube Live stream name or the
            Facebook stream key.
        status (str, Optional): The status of the RTMP stream.
    """

    server_url: Optional[str] = Field(None, validation_alias='serverUrl')
    stream_name: Optional[str] = Field(None, validation_alias='streamName')
    status: Optional[str] = None


class BroadcastUrls(BaseModel):
    """Model for URLs for a broadcast.

    Args:
        hls (str, Optional): URL for the HLS broadcast.
        hls_status (str, Optional): The status of the HLS broadcast.
        rtmp (list[str], Optional): An array of objects that include information on each of the RTMP streams.
    """

    hls: Optional[str] = None
    hls_status: Optional[str] = Field(None, validation_alias='hlsStatus')
    rtmp: Optional[list[RtmpStream]] = None


class HlsSettings(BaseModel):
    """Model for HLS settings for a broadcast.

    Args:
        dvr (bool, Optional): Whether the broadcast supports DVR.
        low_latency (bool, Optional): Whether the broadcast is low latency.
    """

    dvr: Optional[bool] = None
    low_latency: Optional[bool] = Field(None, validation_alias='lowLatency')


class BroadcastSettings(BaseModel):
    """Model for settings for a broadcast.

    Args:
        hls (HlsSettings, Optional): HLS settings for the broadcast.
    """

    hls: Optional[HlsSettings] = None


class Broadcast(BaseModel):
    """Model for a broadcast.

    Args:
        id (str, Optional): The broadcast ID.
        session_id (str, Optional): The video session ID.
        multi_broadcast_tag (str, Optional): The unique tag for simultaneous broadcasts
            (if one was set).
        application_id (str, Optional): The Vonage application ID.
        created_at (int, Optional): The timestamp when the broadcast started, expressed
            in milliseconds since the Unix epoch.
        updated_at (int, Optional): The timestamp when the broadcast was last updated,
            expressed in milliseconds since the Unix epoch.
        max_duration (int, Optional): The maximum duration of the broadcast in seconds.
        max_bitrate (int, Optional): The maximum bitrate of the broadcast.
        broadcast_urls (BroadcastUrls, Optional): An object containing details about the
            HLS and RTMP broadcasts.
        settings (BroadcastHls, Optional): The HLS output settings.
        resolution (VideoResolution, Optional): The resolution of the broadcast.
        has_audio (bool, Optional): Whether the broadcast includes audio.
        has_video (bool, Optional): Whether the broadcast includes video.
        stream_mode (StreamMode, Optional): Whether streams included in the broadcast are
            selected automatically (`auto`, the default) or manually (`manual`).
        status (str, Optional): The status of the broadcast.
        streams (list[VideoStream], Optional): An array of objects corresponding to
            streams currently being broadcast. This is only set for a broadcast with
            the status set to "started" and the `stream_mode` set to "manual".
    """

    id: Optional[str] = None
    session_id: Optional[str] = Field(None, validation_alias='sessionId')
    multi_broadcast_tag: Optional[str] = Field(None, validation_alias='multiBroadcastTag')
    application_id: Optional[str] = Field(None, validation_alias='applicationId')
    created_at: Optional[int] = Field(None, validation_alias='createdAt')
    updated_at: Optional[int] = Field(None, validation_alias='updatedAt')
    max_duration: Optional[int] = Field(None, validation_alias='maxDuration')
    max_bitrate: Optional[int] = Field(None, validation_alias='maxBitrate')
    broadcast_urls: Optional[BroadcastUrls] = Field(
        None, validation_alias='broadcastUrls'
    )
    settings: Optional[BroadcastSettings] = None
    resolution: Optional[VideoResolution] = None
    has_audio: Optional[bool] = Field(None, validation_alias='hasAudio')
    has_video: Optional[bool] = Field(None, validation_alias='hasVideo')
    stream_mode: Optional[StreamMode] = Field(None, validation_alias='streamMode')
    status: Optional[str] = None
    streams: Optional[list[VideoStream]] = None


class BroadcastOutputSettings(BaseModel):
    """Model for output options for a broadcast. You must specify at least one output
    option.

    Args:
        hls (BroadcastHls, Optional): HLS output settings.
        rtmp (list[BroadcastRtmp], Optional): RTMP output settings.

    Raises:
        InvalidOutputOptionsError: If neither HLS nor RTMP output options are set.
    """

    hls: Optional[BroadcastHls] = None
    rtmp: Optional[list[BroadcastRtmp]] = None

    @model_validator(mode='after')
    def validate_outputs(self):
        if self.hls is None and self.rtmp is None:
            raise InvalidOutputOptionsError(
                'You must specify at least one output option.'
            )
        return self


class CreateBroadcastRequest(BaseModel):
    """Model for creating a broadcast.

    Args:
        session_id (str): The session ID of a Vonage Video session.
        layout (Layout, Optional): Layout options for the broadcast.
        max_duration (int, Optional): The maximum duration of the broadcast in seconds.
        outputs (Outputs): Output options for the broadcast. This object defines the types of
            broadcast streams you want to start (both HLS and RTMP). You can include HLS, RTMP,
            or both as broadcast streams. If you include RTMP streaming, you can specify up
            to five target RTMP streams (or just one). Vonage streams the session to each RTMP
            URL you specify. Note that Vonage Video live streaming supports RTMP and RTMPS.
        resolution (VideoResolution, Optional): The resolution of the broadcast.
        stream_mode (StreamMode, Optional): Whether streams included in the broadcast are selected
            automatically ("auto", the default) or manually ("manual").
        multi_broadcast_tag (str, Optional): Set this to support recording multiple broadcasts
            for the same session simultaneously. Set this to a unique string for each simultaneous
            broadcast of an ongoing session. If you do not specify a unique multiBroadcastTag,
            you can only record one broadcast at a time for a given session.
        max_bitrate (int, Optional): The maximum bitrate of the broadcast, in bits per second.
    """

    session_id: str = Field(..., serialization_alias='sessionId')
    layout: Optional[ComposedLayout] = None
    max_duration: Optional[int] = Field(
        None, ge=60, le=36000, serialization_alias='maxDuration'
    )
    outputs: BroadcastOutputSettings
    resolution: Optional[VideoResolution] = None
    stream_mode: Optional[StreamMode] = Field(None, serialization_alias='streamMode')
    multi_broadcast_tag: Optional[str] = Field(
        None, serialization_alias='multiBroadcastTag'
    )
    max_bitrate: Optional[int] = Field(
        None, ge=100_000, le=6_000_000, serialization_alias='maxBitrate'
    )



from time import time
from typing import Literal, Optional
from uuid import uuid4

from pydantic import BaseModel, Field, field_validator, model_validator

from ..errors import TokenExpiryError
from .enums import TokenRole


class TokenOptions(BaseModel):
    """Options for generating a token for the Vonage Video API.

    Args:
        session_id (str): The session ID.
        role (TokenRole): The role of the token. Defaults to 'publisher'.
        connection_data (str): The connection data for the token.
        initial_layout_class_list (list[str]): The initial layout class list for the token.
        exp (int): The expiry date for the token. Defaults to 15 minutes from the current time.
        jti (Union[UUID, str]): The JWT ID for the token. Defaults to a new UUID.
        iat (float): The time the token was issued. Defaults to the current time.
        subject (str): The subject of the token. Defaults to 'video'.
        scope (str): The scope of the token. Defaults to 'session.connect'.
        acl (dict): The access control list for the token. NOTE: Do not change this value.

    Raises:
        TokenExpiryError: If the expiry date is in the past or more than 30 days in the future.
    """

    session_id: str
    role: Optional[TokenRole] = TokenRole.PUBLISHER
    connection_data: Optional[str] = None
    initial_layout_class_list: Optional[list[str]] = None
    exp: Optional[int] = None
    jti: str = Field(default_factory=lambda: str(uuid4()))
    iat: int = Field(default_factory=lambda: int(time()))
    subject: Literal['video'] = 'video'
    scope: Literal['session.connect'] = 'session.connect'
    acl: dict = {'paths': {'/session/**': {}}}

    @field_validator('exp')
    @classmethod
    def validate_exp(cls, v: int):
        now = int(time())
        if v < now:
            raise TokenExpiryError('Token expiry date must be in the future.')
        if v > now + 3600 * 24 * 30:
            raise TokenExpiryError(
                'Token expiry date must be less than 30 days from now.'
            )
        return v

    @model_validator(mode='after')
    def set_exp(self):
        if self.exp is None:
            self.exp = self.iat + 15 * 60
        return self

    @model_validator(mode='after')
    def enforce_acl_default_value(self):
        self.acl = {'paths': {'/session/**': {}}}
        return self



from .archive import Archive, CreateArchiveRequest, ListArchivesFilter, Transcription
from .audio_connector import (
    AudioConnectorData,
    AudioConnectorOptions,
    AudioConnectorWebSocket,
)
from .broadcast import (
    Broadcast,
    BroadcastHls,
    BroadcastOutputSettings,
    BroadcastRtmp,
    BroadcastSettings,
    BroadcastUrls,
    CreateBroadcastRequest,
    HlsSettings,
    ListBroadcastsFilter,
    RtmpStream,
)
from .captions import CaptionsData, CaptionsOptions
from .common import AddStreamRequest, ComposedLayout, ListVideoFilter, VideoStream
from .enums import (
    ArchiveMode,
    ArchiveStatus,
    AudioSampleRate,
    ExperienceComposerStatus,
    LanguageCode,
    LayoutType,
    MediaMode,
    OutputMode,
    P2pPreference,
    StreamMode,
    TokenRole,
    VideoResolution,
)
from .experience_composer import (
    ExperienceComposer,
    ExperienceComposerOptions,
    ExperienceComposerProperties,
    ListExperienceComposersFilter,
)
from .session import SessionOptions, VideoSession
from .signal import SignalData
from .sip import InitiateSipRequest, SipAuth, SipCall, SipOptions
from .stream import StreamInfo, StreamLayout, StreamLayoutOptions
from .token import TokenOptions

__all__ = [
    "AudioConnectorData",
    "AudioConnectorOptions",
    "AudioConnectorWebSocket",
    "Archive",
    "ListArchivesFilter",
    "Transcription",
    "CreateArchiveRequest",
    "Broadcast",
    "BroadcastSettings",
    "BroadcastUrls",
    "HlsSettings",
    "ListBroadcastsFilter",
    "BroadcastHls",
    "RtmpStream",
    "BroadcastRtmp",
    "CreateBroadcastRequest",
    "BroadcastOutputSettings",
    "CaptionsData",
    "CaptionsOptions",
    "ComposedLayout",
    "ListVideoFilter",
    "VideoStream",
    "AddStreamRequest",
    "ArchiveMode",
    "AudioSampleRate",
    "LanguageCode",
    "MediaMode",
    "P2pPreference",
    "TokenRole",
    "VideoResolution",
    "ExperienceComposerStatus",
    "OutputMode",
    "StreamMode",
    "LayoutType",
    "ArchiveStatus",
    "ExperienceComposer",
    "ExperienceComposerOptions",
    "ExperienceComposerProperties",
    "ListExperienceComposersFilter",
    "SessionOptions",
    "VideoSession",
    "SignalData",
    "SipOptions",
    "SipAuth",
    "SipCall",
    "InitiateSipRequest",
    "StreamInfo",
    "StreamLayout",
    "StreamLayoutOptions",
    "TokenOptions",
]



from typing import Optional

from pydantic import BaseModel, Field


class StreamInfo(BaseModel):
    """The stream information.

    Args:
        id (str): The stream ID.
        video_type (str): Set to "camera", "screen", or "custom". A "screen" video uses
            screen sharing on the publisher as the video source; a "custom" video is
            published by a web client using an HTML VideoTrack element as the video
            source.
        name (str): An array of the layout classes for the stream.
        layout_class_list (list[str]): An array of the layout classes for the stream.
    """

    id: Optional[str] = Field(None, validation_alias='id')
    video_type: Optional[str] = Field(None, validation_alias='videoType')
    name: Optional[str] = Field(None, validation_alias='name')
    layout_class_list: Optional[list[str]] = Field(
        None, validation_alias='layoutClassList'
    )


class StreamLayout(BaseModel):
    """The stream layout.

    Args:
        id (str): The stream ID.
        layout_class_list (list[str]): An array of the layout classes for the stream.
    """

    id: str
    layout_class_list: list[str] = Field(..., serialization_alias='layoutClassList')


class StreamLayoutOptions(BaseModel):
    """The options for the stream layout.

    Args:
        items (list[[StreamLayout]]): An array of the stream layout items. Each item is a StreamLayout
            object. See StreamLayout.
    """

    items: list[StreamLayout]



from typing import Optional

from pydantic import BaseModel, Field, model_validator
from vonage_video.errors import IndividualArchivePropertyError, NoAudioOrVideoError
from vonage_video.models.common import ComposedLayout, ListVideoFilter, VideoStream
from vonage_video.models.enums import (
    ArchiveStatus,
    OutputMode,
    StreamMode,
    VideoResolution,
)


class ListArchivesFilter(ListVideoFilter):
    """Model with filters for listing archives.

    Args:
        offset (int, Optional): The offset.
        page_size (int, Optional): The number of archives to return per page.
        session_id (str, Optional): The session ID of a Vonage Video session.
    """

    session_id: Optional[str] = None


class Transcription(BaseModel):
    """Model for transcription options for an archive.

    Args:
        status (str, Optional): The status of the transcription.
        reason (str, Optional): May give a brief reason for the transcription status.
    """

    status: Optional[str] = None
    reason: Optional[str] = None


class Archive(BaseModel):
    """Model for an archive.

    Args:
        id (str, Optional): The unique archive ID.
        status (ArchiveStatus, Optional): The status of the archive.
        name (str, Optional): The name of the archive.
        reason (str, Optional): May give a brief reason for the archive status.
        session_id (str, Optional): The session ID of the Vonage Video session.
        application_id (str, Optional): The Vonage application ID.
        created_at (int, Optional): The timestamp when the archive when the archive
            started recording, expressed in milliseconds since the Unix epoch.
        size (int, Optional): The size of the archive.
        duration (int, Optional): The duration of the archive in seconds.
            For archives that have are being recorded, this value is set to 0.
        output_mode (OutputMode, Optional): The output mode of the archive.
        stream_mode (StreamMode, Optional): Whether streams included in the archive
            are selected automatically (`auto`, the default) or manually (`manual`).
        has_audio (bool, Optional): Whether the archive will record audio.
        has_video (bool, Optional): Whether the archive will record video.
        has_transcription (bool, Optional): Whether audio will be transcribed.
        sha256_sum (str, Optional): The SHA-256 hash of the archive.
        password (str, Optional): The password for the archive.
        updated_at (int, Optional): The timestamp when the archive was last updated,
            expressed in milliseconds since the Unix epoch.
        multi_archive_tag (str, Optional): Set this to support recording multiple
            archives for the same session simultaneously. Set this to a unique string
            for each simultaneous archive of an ongoing session.
        event (str, Optional): The event that triggered the response.
        resolution (VideoResolution, Optional): The resolution of the archive.
        streams (list[VideoStream], Optional): The streams in the archive.
        url (str, Optional): The download URL of the available archive file.
            This is only set for an archive with the status set to `available`.
        transcription (Transcription, Optional): Transcription options for the archive.
        max_bitrate (int, Optional): The maximum video bitrate of the archive, in bits per
            second. This is only valid for composed archives.
    """

    id: Optional[str] = None
    status: Optional[ArchiveStatus] = None
    name: Optional[str] = None
    reason: Optional[str] = None
    session_id: Optional[str] = Field(None, validation_alias='sessionId')
    application_id: Optional[str] = Field(None, validation_alias='applicationId')
    created_at: Optional[int] = Field(None, validation_alias='createdAt')
    size: Optional[int] = None
    duration: Optional[int] = None
    output_mode: Optional[OutputMode] = Field(None, validation_alias='outputMode')
    stream_mode: Optional[StreamMode] = Field(None, validation_alias='streamMode')
    has_audio: Optional[bool] = Field(None, validation_alias='hasAudio')
    has_video: Optional[bool] = Field(None, validation_alias='hasVideo')
    has_transcription: Optional[bool] = Field(None, validation_alias='hasTranscription')
    sha256_sum: Optional[str] = Field(None, validation_alias='sha256sum')
    password: Optional[str] = None
    updated_at: Optional[int] = Field(None, validation_alias='updatedAt')
    multi_archive_tag: Optional[str] = Field(None, validation_alias='multiArchiveTag')
    event: Optional[str] = None
    resolution: Optional[VideoResolution] = None
    streams: Optional[list[VideoStream]] = None
    url: Optional[str] = None
    transcription: Optional[Transcription] = None
    max_bitrate: Optional[int] = Field(None, validation_alias='maxBitrate')


class CreateArchiveRequest(BaseModel):
    """Model for creating an archive.

    Args:
        session_id (str): The session ID of a Vonage Video session.
        has_audio (bool, Optional): Whether the archive should include audio.
        has_video (bool, Optional): Whether the archive should include video.
        layout (Layout, Optional): Layout options for the archive.
        multi_archive_tag (str, Optional): Set this to support recording multiple archives for the same session simultaneously.
            Set this to a unique string for each simultaneous archive of an ongoing session.
            You must also set this option when manually starting an archive in a session that is automatically archived.
            If you do not specify a unique multiArchiveTag, you can only record one archive at a time for a given session.
        name (str, Optional): The name of the archive.
        output_mode (OutputMode, Optional): Whether all streams in the archive are recorded to a
        single file ("composed", the default) or to individual files ("individual").
        resolution (VideoResolution, Optional): The resolution of the archive.
        stream_mode (StreamMode, Optional): Whether streams included in the archive are selected
            automatically ("auto", the default) or manually ("manual").
        max_bitrate (int, Optional): The maximum video bitrate of the archive, in bits per
            second. This is only valid for composed archives.
    Raises:
        NoAudioOrVideoError: If neither `has_audio` nor `has_video` is set.
        IndividualArchivePropertyError: If `resolution` or `layout` is set for individual archives
            or if `has_transcription` is set for composed archives.
    """

    session_id: str = Field(..., serialization_alias='sessionId')
    has_audio: Optional[bool] = Field(None, serialization_alias='hasAudio')
    has_video: Optional[bool] = Field(None, serialization_alias='hasVideo')
    has_transcription: Optional[bool] = Field(
        None, serialization_alias='hasTranscription'
    )
    layout: Optional[ComposedLayout] = None
    multi_archive_tag: Optional[str] = Field(None, serialization_alias='multiArchiveTag')
    name: Optional[str] = None
    output_mode: Optional[OutputMode] = Field(None, serialization_alias='outputMode')
    resolution: Optional[VideoResolution] = None
    stream_mode: Optional[StreamMode] = Field(None, serialization_alias='streamMode')
    max_bitrate: Optional[int] = Field(
        None, ge=100_000, le=6_000_000, serialization_alias='maxBitrate'
    )

    @model_validator(mode='after')
    def validate_audio_or_video(self):
        if self.has_audio is False and self.has_video is False:
            raise NoAudioOrVideoError(
                'One of `has_audio` or `has_video` must be included.'
            )
        return self

    @model_validator(mode='after')
    def no_layout_or_resolution_for_individual_archives(self):
        if self.output_mode == OutputMode.INDIVIDUAL and self.resolution is not None:
            raise IndividualArchivePropertyError(
                'The `resolution` property cannot be set for `archive_mode: \'individual\'`.'
            )
        if self.output_mode == OutputMode.INDIVIDUAL and self.layout is not None:
            raise IndividualArchivePropertyError(
                'The `layout` property cannot be set for `archive_mode: \'individual\'`.'
            )
        return self

    @model_validator(mode='after')
    def transcription_only_for_individual_archives(self):
        if self.output_mode == OutputMode.COMPOSED and self.has_transcription is True:
            raise IndividualArchivePropertyError(
                'The `has_transcription` property can only be set for `archive_mode: \'individual\'`.'
            )
        return self



from . import errors, models  # Import models to access the module directly
from .models import *  # Need this to directly expose data models
from .video import Video

__all__ = ['Video', 'errors', 'models']



from typing import Optional

from pydantic import BaseModel, Field
from vonage_video.models.enums import ExperienceComposerStatus, VideoResolution


class ExperienceComposerProperties(BaseModel):
    """Model with properties for an Experience Composer session.

    Args:
        name (str): The name of the composed output stream which is published to the session.
    """

    name: str = Field(..., min_length=1, max_length=200)


class ExperienceComposerOptions(BaseModel):
    """The options for the Experience Composer.

    Args:
        session_id (str): The session ID of the Vonage Video session you are working with.
        token (str): A valid Vonage Video JWT with a Publisher role and (optionally) connection data to be associated with the output stream.
        url (str, Optional): A publicly reachable URL controlled by the customer and capable of generating the content to be rendered without user intervention.
        max_duration (int, Optional): The maximum duration.
        resolution (ExperienceComposerResolution, Optional): The resolution of the Experience Composer stream.
        properties (ExperienceComposerProperties, Optional): The initial configuration of Publisher properties for the composed output stream.
    """

    session_id: str = Field(..., serialization_alias='sessionId')
    token: str
    url: str = Field(..., min_length=15, max_length=2048)
    max_duration: Optional[int] = Field(
        None, ge=60, le=36000, serialization_alias='maxDuration'
    )
    resolution: Optional[VideoResolution] = None
    properties: Optional[ExperienceComposerProperties] = None


class ExperienceComposer(BaseModel):
    """Model with data describing an Experience Composer session.

    Args:
        id (str, Optional): The unique ID for the Experience Composer.
        session_id (str, Optional): The session ID of the Vonage Video session you are working with.
        application_id (str, Optional): The Vonage application ID.
        created_at (int, Optional): The time the Experience Composer started, expressed in milliseconds since the Unix epoch (January 1, 1970, 00:00:00 UTC).
        callback_url (str, Optional): The callback URL for Experience Composer events (if one was set).
        updated_at (int, Optional): The UNIX timestamp when the Experience Composer status was last updated.
        name (str, Optional): The name of the composed output stream which is published to the session.
        url (str, Optional): A publicly reachable URL controlled by the customer and capable of generating the content to be rendered without user intervention.
        resolution (ExperienceComposerResolution, Optional): The resolution of the Experience Composer stream.
        status (ExperienceComposerStatus, Optional): The status.
        stream_id (str, Optional): The ID of the composed stream being published.
        reason (str, Optional): The reason for the status change.
    """

    id: Optional[str] = None
    session_id: Optional[str] = Field(None, validation_alias='sessionId')
    application_id: Optional[str] = Field(None, validation_alias='applicationId')
    created_at: Optional[int] = Field(None, validation_alias='createdAt')
    callback_url: Optional[str] = Field(None, validation_alias='callbackUrl')
    updated_at: Optional[int] = Field(None, validation_alias='updatedAt')
    name: Optional[str] = None
    url: Optional[str] = None
    resolution: Optional[VideoResolution] = None
    status: Optional[ExperienceComposerStatus] = None
    stream_id: Optional[str] = Field(None, validation_alias='streamId')
    reason: Optional[str] = None


class ListExperienceComposersFilter(BaseModel):
    """Request object for filtering Experience Composers associated with the specific
    Vonage application.

    Args:
        offset (int, Optional): The offset.
        page_size (int, Optional): The number of Experience Composers to return.
    """

    offset: Optional[int] = None
    page_size: Optional[int] = Field(100, serialization_alias='count')
